@@ -105,6 +105,7 @@ ffmpeg_thumbnailer_provider_get_thumbnailers (TumblerThumbnailerProvider *provid
   {
     "video/jpeg",
     "video/mp4",
+    "video/mp2t",
     "video/mpeg",
     "video/quicktime",
     "video/x-ms-asf",
@@ -113,6 +114,7 @@ ffmpeg_thumbnailer_provider_get_thumbnailers (TumblerThumbnailerProvider *provid
     "video/x-msvideo",
     "video/x-flv",
     "application/x-flash-video",
+    "application/vnd.rn-realmedia",
     "video/3gpp",
     "video/x-matroska",
     "video/ogg",
