@@ -34,6 +34,7 @@
 
 #include <tumblerd/tumbler-group-scheduler.h>
 #include <tumblerd/tumbler-scheduler.h>
+#include <tumblerd/tumbler-utils.h>
 
 
 
@@ -93,7 +94,7 @@ struct _TumblerGroupScheduler
   GObject __parent__;
 
   GThreadPool *pool;
-  GMutex      *mutex;
+  TUMBLER_MUTEX (mutex);
   GList       *requests;
   guint        group;
   gboolean     prioritized;
@@ -151,7 +152,7 @@ tumbler_group_scheduler_iface_init (TumblerSchedulerIface *iface)
 static void
 tumbler_group_scheduler_init (TumblerGroupScheduler *scheduler)
 {
-  scheduler->mutex = g_mutex_new ();
+  tumbler_mutex_create (scheduler->mutex);
   scheduler->requests = NULL;
 
   /* Note that unless we convert this boolean to a TLS (thread-local), that
@@ -187,7 +188,7 @@ tumbler_group_scheduler_finalize (GObject *object)
   g_free (scheduler->name);
 
   /* destroy the mutex */
-  g_mutex_free (scheduler->mutex);
+  tumbler_mutex_free (scheduler->mutex);
 
   (*G_OBJECT_CLASS (tumbler_group_scheduler_parent_class)->finalize) (object);
 }
@@ -246,7 +247,7 @@ tumbler_group_scheduler_push (TumblerScheduler        *scheduler,
   g_return_if_fail (TUMBLER_IS_GROUP_SCHEDULER (scheduler));
   g_return_if_fail (request != NULL);
 
-  g_mutex_lock (group_scheduler->mutex);
+  tumbler_mutex_lock (group_scheduler->mutex);
   
   /* gain ownership over the requests (sets request->scheduler) */
   tumbler_scheduler_take_request (scheduler, request);
@@ -257,7 +258,7 @@ tumbler_group_scheduler_push (TumblerScheduler        *scheduler,
   /* enqueue the request in the pool */
   g_thread_pool_push (group_scheduler->pool, request, NULL);
 
-  g_mutex_unlock (group_scheduler->mutex);
+  tumbler_mutex_unlock (group_scheduler->mutex);
 }
 
 
@@ -272,14 +273,14 @@ tumbler_group_scheduler_dequeue (TumblerScheduler *scheduler,
   g_return_if_fail (TUMBLER_IS_GROUP_SCHEDULER (scheduler));
   g_return_if_fail (handle != 0);
 
-  g_mutex_lock (group_scheduler->mutex);
+  tumbler_mutex_lock (group_scheduler->mutex);
 
   /* dequeue all requests (usually only one) with this handle */
   g_list_foreach (group_scheduler->requests, 
                   (GFunc) tumbler_group_scheduler_dequeue_request, 
                   GUINT_TO_POINTER (handle));
 
-  g_mutex_unlock (group_scheduler->mutex);
+  tumbler_mutex_unlock (group_scheduler->mutex);
 }
 
 
@@ -301,7 +302,7 @@ tumbler_group_scheduler_cancel_by_mount (TumblerScheduler *scheduler,
   /* determine the root mount point */
   mount_point = g_mount_get_root (mount);
 
-  g_mutex_lock (group_scheduler->mutex);
+  tumbler_mutex_lock (group_scheduler->mutex);
 
   /* iterate over all requests */
   for (iter = group_scheduler->requests; iter != NULL; iter = iter->next)
@@ -323,7 +324,7 @@ tumbler_group_scheduler_cancel_by_mount (TumblerScheduler *scheduler,
         }
     }
 
-  g_mutex_unlock (group_scheduler->mutex);
+  tumbler_mutex_unlock (group_scheduler->mutex);
 
   /* release the mount point */
   g_object_unref (mount_point);
@@ -438,27 +439,27 @@ tumbler_group_scheduler_thread (gpointer data,
                          request->origin);
 
   /* finish the request if it was dequeued */
-  g_mutex_lock (scheduler->mutex);
+  tumbler_mutex_lock (scheduler->mutex);
   if (request->dequeued)
     {
       tumbler_group_scheduler_finish_request (scheduler, request);
-      g_mutex_unlock (scheduler->mutex);
+      tumbler_mutex_unlock (scheduler->mutex);
       return;
     }
-  g_mutex_unlock (scheduler->mutex);
+  tumbler_mutex_unlock (scheduler->mutex);
 
   /* process URI by URI */
   for (n = 0; n < request->length; ++n)
     {
       /* finish the request if it was dequeued */
-      g_mutex_lock (scheduler->mutex);
+      tumbler_mutex_lock (scheduler->mutex);
       if (request->dequeued)
         {
           tumbler_group_scheduler_finish_request (scheduler, request);
-          g_mutex_unlock (scheduler->mutex);
+          tumbler_mutex_unlock (scheduler->mutex);
           return;
         }
-      g_mutex_unlock (scheduler->mutex);
+      tumbler_mutex_unlock (scheduler->mutex);
 
       /* ignore the the URI if has been cancelled already */
       if (g_cancellable_is_cancelled (request->cancellables[n]))
@@ -537,13 +538,13 @@ tumbler_group_scheduler_thread (gpointer data,
       n = GPOINTER_TO_INT (lp->data);
 
       /* finish the request if it was dequeued */
-      g_mutex_lock (scheduler->mutex);
+      tumbler_mutex_lock (scheduler->mutex);
       if (request->dequeued)
         {
           tumbler_group_scheduler_finish_request (scheduler, request);
           return;
         }
-      g_mutex_unlock (scheduler->mutex);
+      tumbler_mutex_unlock (scheduler->mutex);
 
       /* connect to the error signal of the thumbnailer */
       g_signal_connect (request->thumbnailers[n], "error", 
@@ -566,7 +567,7 @@ tumbler_group_scheduler_thread (gpointer data,
                                             0, 0, NULL, NULL, request);
     }
 
-  g_mutex_lock (scheduler->mutex);
+  tumbler_mutex_lock (scheduler->mutex);
 
   /* We emit all the errors and ready signals together in order to 
    * reduce the overall D-Bus traffic */
@@ -649,7 +650,7 @@ tumbler_group_scheduler_thread (gpointer data,
   /* notify others that we're finished processing the request */
   tumbler_group_scheduler_finish_request (scheduler, request);
 
-  g_mutex_unlock (scheduler->mutex);
+  tumbler_mutex_unlock (scheduler->mutex);
 }
 
 
