@@ -274,13 +274,19 @@ tumbler_registry_lookup (TumblerRegistry *registry,
 
   thumbnailer = g_hash_table_lookup (registry->preferred_thumbnailers, hash_key);
   if (thumbnailer != NULL)
-    available = g_list_prepend (available, g_object_ref (thumbnailer));
+    {
+      g_return_val_if_fail (TUMBLER_IS_THUMBNAILER (available), NULL);
+      available = g_list_prepend (available, g_object_ref (thumbnailer));
+    }
 
   list = g_hash_table_lookup (registry->thumbnailers, hash_key);
   if (list != NULL)
     {
       for (lp = *list; lp != NULL; lp = lp->next)
-        available = g_list_prepend (available, g_object_ref (lp->data));
+        {
+          g_return_val_if_fail (TUMBLER_IS_THUMBNAILER (lp->data), NULL);
+          available = g_list_prepend (available, g_object_ref (lp->data));
+        }
     }
 
   return g_list_reverse (available);
@@ -294,6 +300,8 @@ tumbler_registry_get_file_size (GFile *gfile)
   GFileInfo *file_info;
   gint64     size = 0;
 
+  g_return_val_if_fail (G_IS_FILE (gfile), 0);
+
   file_info = g_file_query_info (gfile,
                                  G_FILE_ATTRIBUTE_STANDARD_SIZE,
                                  G_FILE_QUERY_INFO_NONE, NULL, NULL);
@@ -445,7 +453,7 @@ tumbler_registry_get_thumbnailer_array (TumblerRegistry    *registry,
   g_return_val_if_fail (TUMBLER_IS_REGISTRY (registry), NULL);
   g_return_val_if_fail (infos != NULL, NULL);
 
-  for (length = 0; infos != NULL && infos[length] != NULL; ++length);
+  tumbler_mutex_lock (registry->mutex);
 
   /* allocate the thumbnailer array */
   thumbnailers = g_new0 (TumblerThumbnailer *, length + 1);
@@ -453,7 +461,7 @@ tumbler_registry_get_thumbnailer_array (TumblerRegistry    *registry,
   /* iterate over all URIs */
   for (n = 0; n < length; ++n)
     {
-      tumbler_mutex_lock (registry->mutex);
+      g_assert (TUMBLER_IS_FILE_INFO (infos[n]));
 
       /* reset */
       file_size = 0;
@@ -495,13 +503,13 @@ tumbler_registry_get_thumbnailer_array (TumblerRegistry    *registry,
       g_object_unref (gfile);
       g_list_foreach (list, (GFunc) g_object_unref, NULL);
       g_list_free (list);
-
-      tumbler_mutex_unlock (registry->mutex);
     }
 
   /* NULL-terminate the array */
   thumbnailers[n] = NULL;
 
+  tumbler_mutex_unlock (registry->mutex);
+
   return thumbnailers;
 }
 
